﻿namespace TestLibrary
{
    /// <summary>
    /// 레벨 번호 타입
    /// </summary>
    public enum LevelNumberType
    {
        /// <summary>
        /// None
        /// </summary>
        NONE = -10,

        /// <summary>
        /// Arabic (1, 2, 3)
        /// </summary>
        ARABIC = 0,

        /// <summary>
        /// Uppercase Roman numeral (I, II, III)
        /// </summary>
        UPPERCASE_ROMAN_NUMERAL = 1,

        /// <summary>
        /// Lowercase Roman numeral (i, ii, iii)
        /// </summary>
        LOWERCASE_ROMAN_NUMERAL = 2,

        /// <summary>
        /// Uppercase letter (A, B, C)
        /// </summary>
        UPPERCASE_LETTER = 3,

        /// <summary>
        /// Lowercase letter (a, b, c)
        /// </summary>
        LOWERCASE_LETTER = 4,

        /// <summary>
        /// Ordinal number (1st, 2nd, 3rd)
        /// </summary>
        ORDINAL_NUMBER = 5,

        /// <summary>
        /// Cardinal text number (One, Two Three)
        /// </summary>
        CARDINAL_TEXT_NUMBER = 6,

        /// <summary>
        /// Ordinal text number (First, Second, Third)
        /// </summary>
        ORDINAL_TEXT_NUMBER = 7,

        /// <summary>
        /// Kanji numbering without the digit character (*dbnum1)
        /// </summary>
        KANJI_NUMBERING_WITHOUT_THE_DIGIT_CHARACTER = 10,

        /// <summary>
        /// Kanji numbering with the digit character (*dbnum2)
        /// </summary>
        KANJI_NUMBERING_WITH_THE_DIGIT_CHARACTER = 11,

        /// <summary>
        /// 46 phonetic katakana characters in "aiueo" order (*aiueo)
        /// </summary>
        _46_PHONETIC_KATAKANA_CHARACTERS_IN_AIUEO_ORDER = 12,

        /// <summary>
        /// 46 phonetic katakana characters in "iroha" order (*iroha)
        /// </summary>
        _46_PHONETIC_KATAKANA_CHARACTERS_IN_IROHA_ORDER = 13,

        /// <summary>
        /// Double byte character
        /// </summary>
        DOUBLE_BYTE_CHARACTER = 14,

        /// <summary>
        /// Single_byte character
        /// </summary>
        SINGLE_BYTE_CHARACTER = 15,

        /// <summary>
        /// Kanji numbering 3 (*dbnum3)
        /// </summary>
        KANJI_NUMBERING_3 = 16,

        /// <summary>
        /// Kanji numbering 4 (*dbnum4)
        /// </summary>
        KANJI_NUMBERING_4 = 17,

        /// <summary>
        /// Circle numbering (*circlenum)
        /// </summary>
        CIRCLE_NUMBERING = 18,

        /// <summary>
        /// Double_byte Arabic numbering
        /// </summary>
        DOUBLE_BYTE_ARABIC_NUMBERING = 19,

        /// <summary>
        /// 46 phonetic double_byte katakana characters (*aiueo*dbchar)
        /// </summary>
        _46_PHONETIC_DOUBLE_BYTE_KATAKANA_CHARACTERS_AIUEO_DBCHAR = 20,

        /// <summary>
        /// 46 phonetic double_byte katakana characters (*iroha*dbchar)
        /// </summary>
        _46_PHONETIC_DOUBLE_BYTE_KATAKANA_CHARACTERS_IROHA_DBCHAR = 21,

        /// <summary>
        /// Arabic with leading zero (01, 02, 03, ..., 10, 11)
        /// </summary>
        ARABIC_WITH_LEADING_ZERO = 22,

        /// <summary>
        /// Bullet (no number at all)
        /// </summary>
        BULLET = 23,

        /// <summary>
        /// Korean numbering 2 (*ganada)
        /// </summary>
        KOREAN_NUMBERING_2 = 24,

        /// <summary>
        /// Korean numbering 1 (*chosung)
        /// </summary>
        KOREAN_NUMBERING_1 = 25,

        /// <summary>
        /// Chinese numbering 1 (*gb1)
        /// </summary>
        CHINESE_NUMBERING_1 = 26,

        /// <summary>
        /// Chinese numbering 2 (*gb2)
        /// </summary>
        CHINESE_NUMBERING_2 = 27,

        /// <summary>
        /// Chinese numbering 3 (*gb3)
        /// </summary>
        CHINESE_NUMBERING_3 = 28,

        /// <summary>
        /// Chinese numbering 4 (*gb4)
        /// </summary>
        CHINESE_NUMBERING_4 = 29,

        /// <summary>
        /// Chinese Zodiac numbering 1 (* zodiac1)
        /// </summary>
        CHINESE_ZODIAC_NUMBERING_1 = 30,

        /// <summary>
        /// Chinese Zodiac numbering 2 (* zodiac2)
        /// </summary>
        CHINESE_ZODIAC_NUMBERING_2 = 31,

        /// <summary>
        /// Chinese Zodiac numbering 3 (* zodiac3)
        /// </summary>
        CHINESE_ZODIAC_NUMBERING_3 = 32,

        /// <summary>
        /// Taiwanese double_byte numbering 1
        /// </summary>
        TAIWANESE_DOUBLE_BYTE_NUMBERING_1 = 33,

        /// <summary>
        /// Taiwanese double_byte numbering 2
        /// </summary>
        TAIWANESE_DOUBLE_BYTE_NUMBERING_2 = 34,

        /// <summary>
        /// Taiwanese double_byte numbering 3
        /// </summary>
        TAIWANESE_DOUBLE_BYTE_NUMBERING_3 = 35,

        /// <summary>
        /// Taiwanese double_byte numbering 4
        /// </summary>
        TAIWANESE_DOUBLE_BYTE_NUMBERING_4 = 36,

        /// <summary>
        /// Chinese double_byte numbering 1
        /// </summary>
        CHINESE_DOUBLE_BYTE_NUMBERING_1 = 37,

        /// <summary>
        /// Chinese double_byte numbering 2
        /// </summary>
        CHINESE_DOUBLE_BYTE_NUMBERING_2 = 38,

        /// <summary>
        /// Chinese double_byte numbering 3
        /// </summary>
        CHINESE_DOUBLE_BYTE_NUMBERING_3 = 39,

        /// <summary>
        /// Chinese double_byte numbering 4
        /// </summary>
        CHINESE_DOUBLE_BYTE_NUMBERING_4 = 40,

        /// <summary>
        /// Korean double_byte numbering 1
        /// </summary>
        KOREAN_DOUBLE_BYTE_NUMBERING_1 = 41,

        /// <summary>
        /// Korean double_byte numbering 2
        /// </summary>
        KOREAN_DOUBLE_BYTE_NUMBERING_2 = 42,

        /// <summary>
        /// Korean double_byte numbering 3
        /// </summary>
        KOREAN_DOUBLE_BYTE_NUMBERING_3 = 43,

        /// <summary>
        /// Korean double_byte numbering 4
        /// </summary>
        KOREAN_DOUBLE_BYTE_NUMBERING_4 = 44,

        /// <summary>
        /// Hebrew non standard decimal
        /// </summary>
        HEBREW_NON_STANDARD_DECIMAL = 45,

        /// <summary>
        /// Arabic Alif Ba Tah
        /// </summary>
        ARABIC_ALIF_BA_TAH = 46,

        /// <summary>
        /// Hebrew Biblical standard
        /// </summary>
        HEBREW_BIBLICAL_STANDARD = 47,

        /// <summary>
        /// Arabic Abjad style
        /// </summary>
        ARABIC_ABJAD_STYLE = 48,

        /// <summary>
        /// No number
        /// </summary>
        NO_NUMBER = 255
    }
}