﻿using System;
using System.Collections;
using System.Diagnostics;

namespace TestLibrary
{
    /// <summary>
    /// RTF 특성 리스트
    /// </summary>
    [Serializable]
    [DebuggerTypeProxy(typeof(RTFInstanceDebugView))]
    public class RTFAttributeList : CollectionBase
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 인덱서 - this[name]

        /// <summary>
        /// 인덱서
        /// </summary>
        /// <param name="name">명칭</param>
        /// <returns>값</returns>
        public int this[string name]
        {
            get
            {
                foreach(RTFAttribute attribute in this)
                {
                    if(attribute.Name == name)
                    {
                        return attribute.Value;
                    }
                }

                return int.MinValue;
            }
            set
            {
                foreach(RTFAttribute attribute in this)
                {
                    if(attribute.Name == name)
                    {
                        attribute.Value = value;

                        return;
                    }
                }

                RTFAttribute newAttribute = new RTFAttribute();

                newAttribute.Name  = name;
                newAttribute.Value = value;

                List.Add(newAttribute);
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - RTFAttributeList()

        /// <summary>
        /// 생성자
        /// </summary>
        public RTFAttributeList()
        {
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 추가하기 - Add(item)

        /// <summary>
        /// 추가하기
        /// </summary>
        /// <param name="item">항목</param>
        /// <returns>인덱스</returns>
        public int Add(RTFAttribute item)
        {
            return List.Add(item);
        }

        #endregion
        #region 추가하기 - Add(name, value)

        /// <summary>
        /// 추가하기
        /// </summary>
        /// <param name="name">명칭</param>
        /// <param name="value">값</param>
        /// <returns>인덱스</returns>
        public int Add(string name, int value)
        {
            RTFAttribute attribute = new RTFAttribute();

            attribute.Name  = name;
            attribute.Value = value;

            return List.Add(attribute);
        }

        #endregion
        #region 제거하기 - Remove(item)

        /// <summary>
        /// 제거하기
        /// </summary>
        /// <param name="item">항목</param>
        public void Remove(RTFAttribute item)
        {
            List.Remove(item);
        }

        #endregion
        #region 제거하기 - Remove(name)

        /// <summary>
        /// 제거하기
        /// </summary>
        /// <param name="name">명칭</param>
        public void Remove(string name)
        {
            for(int i = this.Count - 1; i >= 0; i--)
            {
                RTFAttribute item = (RTFAttribute)this.List[i];

                if(item.Name == name)
                {
                    List.RemoveAt(i);
                }
            }
        }

        #endregion
        #region 항목 구하기 - GetItem(index)

        /// <summary>
        /// 항목 구하기
        /// </summary>
        /// <param name="index">인덱스</param>
        /// <returns>항목</returns>
        public RTFAttribute GetItem(int index)
        {
            return (RTFAttribute)List[index];
        }

        #endregion
        #region 포함 여부 구하기 - Contains(item)

        /// <summary>
        /// 포함 여부 구하기
        /// </summary>
        /// <param name="item">항목</param>
        /// <returns>포함 여부</returns>
        public bool Contains(RTFAttribute item)
        {
            return List.Contains(item);
        }

        #endregion
        #region 포함 여부 구하기 - Contains(name)

        /// <summary>
        /// 포함 여부 구하기
        /// </summary>
        /// <param name="name">명칭</param>
        /// <returns>포함 여부</returns>
        public bool Contains(string name)
        {
            foreach(RTFAttribute attribute in this)
            {
                if(attribute.Name == name)
                {
                    return true;
                }
            }

            return false;
        }

        #endregion
        #region 복제하기 - Clone()

        /// <summary>
        /// 복제하기
        /// </summary>
        /// <returns>복제 리스트</returns>
        public RTFAttributeList Clone()
        {
            RTFAttributeList list = new RTFAttributeList();

            foreach(RTFAttribute item in this)
            {
                RTFAttribute newItem = new RTFAttribute();

                newItem.Name  = item.Name;
                newItem.Value = item.Value;

                list.List.Add(newItem);
            }

            return list;
        }

        #endregion
    }
}