﻿using System.Drawing ;
using System.Drawing.Drawing2D;
using System.ComponentModel;

namespace TestLibrary
{
    /// <summary>
    /// RTF 테두리 스타일
    /// </summary>
    public class RTFBorderStyle
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 왼쪽 여부
        /// </summary>
        private bool left = false;

        /// <summary>
        /// 위쪽 여부
        /// </summary>
        private bool top = false;

        /// <summary>
        /// 오른쪽 여부
        /// </summary>
        private bool right = false;

        /// <summary>
        /// 아래쪽 여부
        /// </summary>
        private bool bottom = false;

        /// <summary>
        /// 두께
        /// </summary>
        private bool thickness = false;

        /// <summary>
        /// 스타일
        /// </summary>
        private DashStyle style = DashStyle.Solid;

        /// <summary>
        /// 색상
        /// </summary>
        private Color color = Color.Black;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Publicㄴ

        #region 왼쪽 여부 - Left

        /// <summary>
        /// 왼쪽 여부
        /// </summary>
        [DefaultValue(false)]
        public bool Left
        {
            get
            {
                return this.left;
            }
            set
            {
                this.left = value;
            }
        }

        #endregion
        #region 위쪽 여부 - Top

        /// <summary>
        /// 위쪽
        /// </summary>
        [DefaultValue(false)]
        public bool Top
        {
            get
            {
                return this.top;
            }
            set
            {
                this.top = value;
            }
        }

        #endregion
        #region 오른쪽 여부 - Right

        /// <summary>
        /// 오른쪽 여부
        /// </summary>
        [DefaultValue(false)]
        public bool Right
        {
            get
            {
                return this.right;
            }
            set
            {
                this.right = value;
            }
        }

        #endregion
        #region 아래쪽 여부 - Bottom

        /// <summary>
        /// 아래쪽 여부
        /// </summary>
        [DefaultValue(false)]
        public bool Bottom
        {
            get
            {
                return this.bottom;
            }
            set
            {
                this.bottom = value;
            }
        }

        #endregion
        #region 두께 - Thickness

        /// <summary>
        /// 두께
        /// </summary>
        [DefaultValue(false)]
        public bool Thickness
        {
            get
            {
                return this.thickness;
            }
            set
            {
                this.thickness = value;
            }
        }

        #endregion
        #region 스타일 - Style

        /// <summary>
        /// 스타일
        /// </summary>
        [DefaultValue(DashStyle.Solid)]
        public DashStyle Style
        {
            get
            {
                return this.style;
            }
            set
            {
                this.style = value;
            }
        }

        #endregion
        #region 색상 - Color

        /// <summary>
        /// 색상
        /// </summary>
        [DefaultValue(typeof(Color), "Black")]
        public Color Color
        {
            get
            {
                return this.color;
            }
            set
            {
                this.color = value;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Publicㄴ

        #region 복제하기 - Clone()

        /// <summary>
        /// 복제하기
        /// </summary>
        /// <returns>RTF 테두리 스타일</returns>
        public RTFBorderStyle Clone()
        {
            RTFBorderStyle stryle = new RTFBorderStyle();

            stryle.left      = this.left;
            stryle.top       = this.top;
            stryle.right     = this.right;
            stryle.bottom    = this.bottom;
            stryle.thickness = this.thickness;
            stryle.style     = this.style;
            stryle.color     = this.color;

            return stryle;
        }

        #endregion
        #region 값 동일 여부 구하기 - EqualsValue(style)

        /// <summary>
        /// 값 동일 여부 구하기
        /// </summary>
        /// <param name="style">스타일</param>
        /// <returns>값 동일 여부</returns>
        public bool EqualsValue(RTFBorderStyle style)
        {
            if(style == this)
            {
                return true;
            }

            if(style == null)
            {
                return false;
            }

            if
            (
                style.bottom    != this.bottom ||
                style.color     != this.color  ||
                style.left      != this.left   ||
                style.right     != this.right  ||
                style.style     != this.style  ||
                style.top       != this.top    ||
                style.thickness != this.thickness
            )
            {
                return false;
            }
            else
            {
                return true;
            }
        }

        #endregion
    }
}