﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Text;

namespace TestLibrary
{
    /// <summary>
    /// RTF DOM 객체
    /// </summary>
    [Serializable]
    public class RTFDOMObject : RTFDOMElement
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 명칭
        /// </summary>
        private string name = null;

        /// <summary>
        /// 클래스명
        /// </summary>
        private string className = null;

        /// <summary>
        /// RTF 객체 타입
        /// </summary>
        private RTFObjectType objectType = RTFObjectType.EMB;

        /// <summary>
        /// 너비
        /// </summary>
        private int width = 0;

        /// <summary>
        /// 높이
        /// </summary>
        private int height = 0;

        /// <summary>
        /// 스케일 X
        /// </summary>
        private int scaleX = 100;

        /// <summary>
        /// 스케일 Y
        /// </summary>
        private int scaleY = 100;

        /// <summary>
        /// 컨텐트 바이트 배열
        /// </summary>
        private byte[] contentByteArray = null;

        /// <summary>
        /// 커스텀 특성 딕셔너리
        /// </summary>
        private Dictionary<string, string> customAttributeDictionary = new Dictionary<string, string>();

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 명칭 - Name

        /// <summary>
        /// 명칭
        /// </summary>
        public string Name
        {
            get
            {
                return this.name;
            }
            set
            {
                this.name = value;
            }
        }

        #endregion
        #region 클래스명 - ClassName

        /// <summary>
        /// 클래스명
        /// </summary>
        public string ClassName
        {
            get
            {
                return this.className;
            }
            set
            {
                this.className = value;
            }
        }

        #endregion
        #region 객체 타입 - ObjectType

        /// <summary>
        /// 객체 타입
        /// </summary>
        public RTFObjectType ObjectType
        {
            get
            {
                return this.objectType;
            }
            set
            {
                this.objectType = value;
            }
        }

        #endregion
        #region 너비 - Width

        /// <summary>
        /// 너비
        /// </summary>
        public int Width
        {
            get
            {
                return this.width;
            }
            set
            {
                this.width = value;
            }
        }

        #endregion
        #region 높이 - Height

        /// <summary>
        /// 높이
        /// </summary>
        public int Height
        {
            get
            {
                return this.height;
            }
            set
            {
                this.height = value;
            }
        }

        #endregion
        #region 스케일 X - ScaleX

        /// <summary>
        /// 스케일 X
        /// </summary>
        public int ScaleX
        {
            get
            {
                return this.scaleX;
            }
            set
            {
                this.scaleX = value;
            }
        }

        #endregion
        #region 스케일 Y - ScaleY

        /// <summary>
        /// 스케일 Y
        /// </summary>
        public int ScaleY
        {
            get
            {
                return this.scaleY;
            }
            set
            {
                this.scaleY = value;
            }
        }

        #endregion
        #region 컨텐트 바이트 배열 - ContentByteArray

        /// <summary>
        /// 컨텐트 바이트 배열
        /// </summary>
        public byte[] ContentByteArray
        {
            get
            {
                return this.contentByteArray;
            }
            set
            {
                this.contentByteArray = value;
            }
        }

        #endregion
        #region 컨텐트 텍스트 - ContentText

        /// <summary>
        /// 컨텐트 텍스트
        /// </summary>
        [Browsable( false )]
        public string ContentText
        {
            get
            {
                if(this.contentByteArray == null || this.contentByteArray.Length == 0)
                {
                    return null;
                }
                else
                {
                    return Encoding.Default.GetString(this.contentByteArray);
                }
            }
        }

        #endregion
        #region 커스텀 특성 딕셔너리 - CustomAttributeDictionary

        /// <summary>
        /// 커스텀 특성 딕셔너리
        /// </summary>
        public Dictionary<string, string> CustomAttributeDictionary
        {
            get
            {
                if(this.customAttributeDictionary == null)
                {
                    this.customAttributeDictionary = new Dictionary<string, string>();
                }

                return this.customAttributeDictionary; 
            }
            set
            {
                this.customAttributeDictionary = value;
            }
        }

        #endregion
        #region 결과 - Result

        /// <summary>
        /// 결과
        /// </summary>
        [DefaultValue(null)]
        public RTFDOMElementContainer Result
        {
            get
            {
                foreach(RTFDOMElement element in ElementList)
                {
                    if(element is RTFDOMElementContainer)
                    {
                        RTFDOMElementContainer container = (RTFDOMElementContainer)element;

                        if(container.Name == RTFConstant.RESULT)
                        {
                            return container;
                        }
                    }
                }

                return null;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - RTFDomObject()

        /// <summary>
        /// 생성자
        /// </summary>
        public RTFDOMObject()
        {
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 문자열 구하기 - ToString()

        /// <summary>
        /// 문자열 구하기
        /// </summary>
        /// <returns>문자열</returns>
        public override string ToString()
        {
            string text = $"Object:{this.Width}*{this.Height}";

            if(this.contentByteArray != null && this.contentByteArray.Length > 0)
            {
                text = $"{text} {Convert.ToDouble(this.contentByteArray.Length / 1024.0).ToString("0.00")}KB";
            }

            return text;
        }

        #endregion
    }
}