﻿using System.Collections.Generic;
using System.Text;

namespace TestLibrary
{
    /// <summary>
    /// RTF 폰트
    /// </summary>
    public class RTFFont
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 인코딩 딕셔너리
        /// </summary>
        private static Dictionary<int, Encoding> _encodingDictionary = null;

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 인덱스
        /// </summary>
        private int index = 0;

        /// <summary>
        /// 폰트명
        /// </summary>
        private string name = null;

        /// <summary>
        /// 문자 세트
        /// </summary>
        private int characterSet = 1;

        /// <summary>
        /// 인코딩
        /// </summary>
        private Encoding encoding = null;

        /// <summary>
        /// NULL 여부
        /// </summary>
        private bool isNull = false;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 인덱스 - Index

        /// <summary>
        /// 인덱스
        /// </summary>
        public int Index
        {
            get
            {
                return this.index;
            }
            set
            {
                this.index = value;
            }
        }

        #endregion
        #region 폰트명 - Name

        /// <summary>
        /// 폰트명
        /// </summary>
        public string Name
        {
            get
            {
                return this.name;
            }
            set
            {
                this.name = value;
            }
        }

        #endregion
        #region 문자 세트 - CharacterSet

        /// <summary>
        /// 문자 세트
        /// </summary>
        public int CharacterSet
        {
            get
            {
                return this.characterSet;
            }
            set
            {
                this.characterSet = value;
                this.encoding = GetEncoding(this.characterSet);
            }
        }

        #endregion
        #region 인코딩 - Encoding

        /// <summary>
        /// 인코딩
        /// </summary>
        public Encoding Encoding
        {
            get 
            {
                return this.encoding;
            }
        }

        #endregion
        #region NULL 여부 - IsNull

        /// <summary>
        /// NULL 여부
        /// </summary>
        public bool IsNull
        {
            get
            {
                return this.isNull;
            }
            set
            {
                this.isNull = value;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - RTFFont(index, name)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="index"></param>
        /// <param name="name"></param>
        public RTFFont(int index, string name)
        {
            this.index = index;
            this.name  = name;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Internal

        #region 문자 세트 구하기 - GetCharset(encoding)

        /// <summary>
        /// 문자 세트 구하기
        /// </summary>
        /// <param name="encoding">인코딩</param>
        /// <returns>문자 세트</returns>
        internal static int GetCharset(Encoding encoding)
        {
            CheckEncodingDictionary();

            foreach(int key in _encodingDictionary.Keys)
            {
                if(_encodingDictionary[key] == encoding)
                {
                    return key;
                }
            }

            return 1;
        }

        #endregion
        #region 인코딩 구하기 - GetEncoding(characterSet)

        /// <summary>
        /// 인코딩 구하기
        /// </summary>
        /// <param name="characterSet">문자 세트</param>
        /// <returns>인코딩</returns>
        internal static Encoding GetEncoding(int characterSet)
        {
            if(characterSet == 0)
            {
                return ANSIEncoding.Instance;
            }
            else if(characterSet == 1)
            {
                return Encoding.Default;
            }
            else
            {
                CheckEncodingDictionary();

                if(_encodingDictionary.ContainsKey(characterSet))
                {
                    return _encodingDictionary[characterSet];
                }
                else
                {
                    return null;
                }
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Private

        #region 인코딩 딕셔너리 체크하기 - CheckEncodingDictionary()

        /// <summary>
        /// 인코딩 딕셔너리 체크하기
        /// </summary>
        private static void CheckEncodingDictionary()
        {
            if(_encodingDictionary == null)
            {
                _encodingDictionary = new Dictionary<int, Encoding>();

              //_encodingDictionary[  0] = ANSIEncoding.Instance;
              //_encodingDictionary[  1] = Encoding.Default;
                _encodingDictionary[ 77] = Encoding.GetEncoding(10000); // Mac ,macintosh
                _encodingDictionary[128] = Encoding.GetEncoding(932  ); // Shift Jis ;ANSI/OEM - Japanese, Shift-JIS
                _encodingDictionary[130] = Encoding.GetEncoding(1361 ); // Johab;Korean (Johab)
                _encodingDictionary[134] = Encoding.GetEncoding(936  ); // GB2312
                _encodingDictionary[136] = Encoding.GetEncoding(10002); // Big5
                _encodingDictionary[161] = Encoding.GetEncoding(1253 ); // Greek
                _encodingDictionary[162] = Encoding.GetEncoding(1254 ); // Turkish
                _encodingDictionary[163] = Encoding.GetEncoding(1258 ); // Vietnamese;ANSI/OEM - Vietnamese
                _encodingDictionary[177] = Encoding.GetEncoding(1255 ); // Hebrw
                _encodingDictionary[178] = Encoding.GetEncoding(864  ); // Arabic
                _encodingDictionary[179] = Encoding.GetEncoding(864  ); // Arabic Traditional
                _encodingDictionary[180] = Encoding.GetEncoding(864  ); // Arabic user
                _encodingDictionary[181] = Encoding.GetEncoding(864  ); // Hebrew user
                _encodingDictionary[186] = Encoding.GetEncoding(775  ); // Baltic
                _encodingDictionary[204] = Encoding.GetEncoding(866  ); // Russian
                _encodingDictionary[222] = Encoding.GetEncoding(874  ); // Thai
                _encodingDictionary[255] = Encoding.GetEncoding(437  ); // OEM
            }
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 복제하기- Clone()

        /// <summary>
        /// 복제하기
        /// </summary>
        /// <returns>RTF 폰트</returns>
        public RTFFont Clone()
        {
            RTFFont font = new RTFFont(this.index , this.name);

            font.index        = this.index;
            font.name         = this.name;
            font.characterSet = this.characterSet;
            font.encoding     = this.encoding;

            return font;
        }

        #endregion
        #region 문자열 구하기 - ToString()

        /// <summary>
        /// 문자열 구하기
        /// </summary>
        /// <returns>문자열</returns>
        public override string ToString()
        {
            return $"{this.index}:{this.name} Charset:{this.characterSet}";
        }

        #endregion
    }
}