using System;
using System.Collections;
using System.Diagnostics;
using System.Text ;

namespace TestLibrary
{
    /// <summary>
    /// RTF Ʈ ̺
    /// </summary>
    [DebuggerDisplay("Count={Count}")]
    [DebuggerTypeProxy(typeof(RTFInstanceDebugView))]
    public class RTFFontTable : CollectionBase
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region ε - this[index]

        /// <summary>
        /// ε
        /// </summary>
        /// <param name="index">ε</param>
        /// <returns>RTF Ʈ</returns>
        public RTFFont this[int index]
        {
            get
            {
                foreach(RTFFont font in this)
                {
                    if(font.Index == index)
                    {
                        return font;
                    }
                }

                return null;
            }
        }

        #endregion
        #region ε - this[name]

        /// <summary>
        /// ε
        /// </summary>
        /// <param name="name">Ʈ</param>
        /// <returns>RTF Ʈ</returns>
        public RTFFont this[string name]
        {
            get
            {
                foreach(RTFFont font in this)
                {
                    if(font.Name == name)
                    {
                        return font;
                    }
                }

                return null;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region  - RTFFontTable()

        /// <summary>
        /// 
        /// </summary>
        public RTFFontTable()
        {
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region ߰ϱ - Add(font)

        /// <summary>
        /// ߰ϱ
        /// </summary>
        /// <param name="font">Ʈ</param>
        public void Add(RTFFont font)
        {
            List.Add(font);
        }

        #endregion
        #region ߰ϱ - Add(index, name, encoding)

        /// <summary>
        /// ߰ϱ
        /// </summary>
        /// <param name="index">ε</param>
        /// <param name="name">Ʈ</param>
        /// <param name="encoding">ڵ</param>
        /// <returns>RTF Ʈ</returns>
        public RTFFont Add(int index, string name, Encoding encoding)
        {
            if(this[name] == null)
            {
                RTFFont font = new RTFFont(index, name);

                if(encoding != null)
                {
                    font.CharacterSet = RTFFont.GetCharset(encoding);
                }

                List.Add(font);

                return font ;
            }

            return this[name];
        }

        #endregion
        #region ߰ϱ - Add(name)

        /// <summary>
        /// ߰ϱ
        /// </summary>
        /// <param name="name">Ʈ</param>
        public RTFFont Add(string name)
        {
            return Add(Count, name, Encoding.Default);
        }

        #endregion
        #region ߰ϱ - Add(name, encoding)

        /// <summary>
        /// ߰ϱ
        /// </summary>
        /// <param name="name">Ʈ</param>
        /// <param name="encoding">ڵ</param>
        /// <returns>RTF Ʈ</returns>
        public RTFFont Add(string name, Encoding encoding)
        {
            return Add(Count, name, encoding);
        }

        #endregion
        #region ϱ - Remove(name)

        /// <summary>
        /// ϱ
        /// </summary>
        /// <param name="name">Ʈ</param>
        public void Remove(string name)
        {
            RTFFont font = this[name];

            if(font != null)
            {
                List.Remove(font);
            }
        }

        #endregion

        #region Ʈ ϱ - GetFontName(index)

        /// <summary>
        /// Ʈ ϱ
        /// </summary>
        /// <param name="index">ε</param>
        /// <returns>Ʈ</returns>
        public string GetFontName(int index)
        {
            RTFFont font = this[index];

            if(font != null)
            {
                return font.Name;
            }
            else
            {
                return null;
            }
        }

        #endregion
        #region ε ϱ - GetIndex(name)

        /// <summary>
        /// ε ϱ
        /// </summary>
        /// <param name="name">Ʈ</param>
        /// <returns>ε</returns>
        public int GetIndex(string name)
        {
            foreach(RTFFont font in this)
            {
                if(font.Name == name)
                {
                    return font.Index;
                }
            }

            return -1 ;
        }

        #endregion

        #region  - Write(writer)

        /// <summary>
        /// 
        /// </summary>
        /// <param name="writer"></param>
        public void Write(RTFWriter writer)
        {
            writer.WriteStartGroup();

            writer.WriteKeyword(RTFConstant.FONTTBL);

            foreach(RTFFont font in this)
            {
                writer.WriteStartGroup();

                writer.WriteKeyword($"f{font.Index}");

                if(font.CharacterSet != 0)
                {
                    writer.WriteKeyword($"fcharset{font.CharacterSet}");
                }

                writer.WriteText(font.Name);

                writer.WriteEndGroup();
            }

            writer.WriteEndGroup();
        }

        #endregion
        #region ϱ - Clone()

        /// <summary>
        /// ϱ
        /// </summary>
        /// <returns>RTF Ʈ ̺</returns>
        public RTFFontTable Clone()
        {
            RTFFontTable fontTable = new RTFFontTable();

            foreach(RTFFont font in this)
            {
                RTFFont newFont = font.Clone();

                fontTable.List.Add(newFont);
            }

            return fontTable;
        }

        #endregion
        #region ڿ ϱ - ToString()

        /// <summary>
        /// ڿ ϱ
        /// </summary>
        /// <returns>ڿ</returns>
        public override string ToString()
        {
            StringBuilder stringBuilder = new StringBuilder();

            foreach(RTFFont font in this)
            {
                stringBuilder.Append(Environment.NewLine);

                stringBuilder.Append($"Index {font.Index}   Name:{font.Name}");
            }

            return stringBuilder.ToString();
        }

        #endregion
    }
}