﻿using System;
using System.Collections;
using System.Diagnostics;

namespace TestLibrary
{
    /// <summary>
    /// RTF 인스턴스 디버그 뷰
    /// </summary>
    public class RTFInstanceDebugView
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 인스턴스
        /// </summary>
        private object instance = null;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 항목 배열 - Items

        /// <summary>
        /// 항목 배열
        /// </summary>
        [DebuggerBrowsable(DebuggerBrowsableState.RootHidden)]
        public object Items
        {
            get
            {
                if(this.instance is IEnumerable)
                {
                    CollectionBase collection = (CollectionBase)this.instance;

                    object[] itemArray = new object[collection.Count];

                    int count = 0;

                    foreach(object item in collection)
                    {
                        itemArray[count] = item;

                        count++;
                    }

                    return itemArray;
                }
                else if(this.instance is RTFColorTable)
                {
                    RTFColorTable table = (RTFColorTable)this.instance;

                    object[] itemArray = new object[table.Count];

                    for(int i = 0; i < table.Count; i++)
                    {
                        itemArray[i] = table[i];
                    }

                    return itemArray;
                }
                else if(this.instance is RTFDocumentInfo)
                {
                    RTFDocumentInfo info = (RTFDocumentInfo)this.instance;

                    return info.ItemArray;
                }
                else
                {
                    return this.instance;
                }
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - RTFInstanceDebugView(instance)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="instance">인스턴스</param>
        public RTFInstanceDebugView(object instance)
        {
            if(instance == null)
            {
                throw new ArgumentNullException("instance");
            }

            this.instance = instance;
        }

        #endregion
    }
}