namespace TestLibrary
{
    /// <summary>
    /// RTF 
    /// </summary>
    public class RTFNode
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Protected

        #region Field

        /// <summary>
        ///  Ÿ
        /// </summary>
        protected RTFNodeType nodeType = RTFNodeType.NONE;

        /// <summary>
        /// θ
        /// </summary>
        protected RTFNodeGroup parent = null;

        /// <summary>
        ///  
        /// </summary>
        protected RTFRawDocument ownerDocument = null ;

        /// <summary>
        /// Ű
        /// </summary>
        protected string keyword = null;

        /// <summary>
        /// Ű   
        /// </summary>
        protected bool hasParameter = false;

        /// <summary>
        /// Ű 
        /// </summary>
        protected int parameter = 0;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region ε - Index

        /// <summary>
        /// ε
        /// </summary>
        public int Index
        {
            get
            {
                if(this.parent == null)
                {
                    return 0;
                }
                else
                {
                    return this.parent.ChildNodeList.GetIndex(this);
                }
            }
        }

        #endregion
        #region  Ÿ - NodeType

        /// <summary>
        ///  Ÿ
        /// </summary>
        public RTFNodeType NodeType
        {
            get
            {
                return this.nodeType;
            }
        }

        #endregion
        #region θ - Parent

        /// <summary>
        /// θ
        /// </summary>
        public virtual RTFNodeGroup Parent
        {
            get
            {
                return this.parent;
            }
            set
            {
                this.parent = value;
            }
        }

        #endregion
        #region   - PreviousNode

        /// <summary>
        ///  
        /// </summary>
        public RTFNode PreviousNode
        {
            get
            {
                if(this.parent != null)
                {
                    int index = this.parent.ChildNodeList.GetIndex(this);

                    if(index > 0)
                    {
                        return this.parent.ChildNodeList[index - 1];
                    }
                }

                return null;
            }
        }

        #endregion
        #region   - NextNode

        /// <summary>
        ///  
        /// </summary>
        public RTFNode NextNode
        {
            get
            {
                if(this.parent != null)
                {
                    int index = this.parent.ChildNodeList.GetIndex(this);

                    if(index >= 0 && index < this.parent.ChildNodeList.Count - 1)
                    {
                        return this.parent.ChildNodeList[index + 1];
                    }
                }

                return null;
            }
        }

        #endregion
        #region ڽ  Ʈ - ChildNodeList

        /// <summary>
        /// ڽ  Ʈ
        /// </summary>
        public virtual RTFNodeList ChildNodeList
        {
            get
            {
                return null;
            }
        }

        #endregion
        #region   - OwnerDocument

        /// <summary>
        ///  
        /// </summary>
        public virtual RTFRawDocument OwnerDocument
        {
            get
            {
                return this.ownerDocument;
            }
            set
            {
                this.ownerDocument = value;

                if(ChildNodeList != null)
                {
                    foreach(RTFNode node in ChildNodeList)
                    {
                        node.OwnerDocument = value;
                    }
                }
            }
        }

        #endregion
        #region Ű - Keyword

        /// <summary>
        /// Ű
        /// </summary>
        public virtual string Keyword
        {
            get
            {
                return this.keyword;
            }
            set
            {
                this.keyword = value;
            }
        }

        #endregion
        #region Ű    - HasParameter

        /// <summary>
        /// Ű   
        /// </summary>
        public virtual bool HasParameter
        {
            get
            {
                return this.hasParameter;
            }
            set
            {
                this.hasParameter = value;
            }
        }

        #endregion
        #region Ű  - Parameter

        /// <summary>
        /// Ű 
        /// </summary>
        public virtual int Parameter
        {
            get
            {
                return this.parameter;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region  - RTFNode()

        /// <summary>
        /// 
        /// </summary>
        public RTFNode()
        {
        }

        #endregion
        #region  - RTFNode(token)

        /// <summary>
        /// 
        /// </summary>
        /// <param name="token">RTF ū</param>
        public RTFNode(RTFToken token)
        {
            this.keyword      = token.Keyword;
            this.hasParameter = token.HasParameter;
            this.parameter    = token.Parameter;

            if(token.TokenType == RTFTokenType.Control)
            {
                this.nodeType = RTFNodeType.CONTROL;
            }
            else if(token.TokenType == RTFTokenType.Control)
            {
                this.nodeType = RTFNodeType.CONTROL;
            }
            else if(token.TokenType == RTFTokenType.Keyword)
            {
                this.nodeType = RTFNodeType.KEYWORD;
            }
            else if(token.TokenType == RTFTokenType.ExtendedKeyword)
            {
                this.nodeType = RTFNodeType.EXTERNAL_KEYWORD;
            }
            else if(token.TokenType == RTFTokenType.Text)
            {
                this.nodeType = RTFNodeType.TEXT;
            }
            else
            {
                this.nodeType = RTFNodeType.TEXT;
            }
        }

        #endregion
        #region  - RTFNode(nodeType, keyword)

        /// <summary>
        /// 
        /// </summary>
        /// <param name="nodeType"> Ÿ</param>
        /// <param name="keyword">Ű</param>
        public RTFNode(RTFNodeType nodeType, string keyword)
        {
            this.nodeType = nodeType;
            this.keyword  = keyword;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region  - Write(writer)

        /// <summary>
        /// 
        /// </summary>
        /// <param name="writer">RTF </param>
        public virtual void Write(RTFWriter writer)
        {
            if(this.nodeType == RTFNodeType.CONTROL || this.nodeType == RTFNodeType.KEYWORD || this.nodeType == RTFNodeType.EXTERNAL_KEYWORD)
            {
                if(this.hasParameter)
                {
                    writer.WriteKeyword(this.keyword + this.parameter, this.nodeType == RTFNodeType.EXTERNAL_KEYWORD);
                }
                else
                {
                    writer.WriteKeyword(this.keyword, this.nodeType == RTFNodeType.EXTERNAL_KEYWORD);
                }
            }
            else if(this.nodeType == RTFNodeType.TEXT)
            {
                writer.WriteText(this.keyword);
            }
        }

        #endregion
    }
}