using System;
using System.Text;

namespace TestLibrary
{
    /// <summary>
    /// RTF 노드 그룹
    /// </summary>
    public class RTFNodeGroup : RTFNode
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Protected

        #region Field

        /// <summary>
        /// 자식 노드 리스트
        /// </summary>
        protected RTFNodeList childNodeList = new RTFNodeList();

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 자식 노드 리스트 - ChildNodeList

        /// <summary>
        /// 자식 노드 리스트
        /// </summary>
        public override RTFNodeList ChildNodeList
        {
            get
            {
                return this.childNodeList;
            }
        }

        #endregion
        #region 첫번째 노드 - FirstNode

        /// <summary>
        /// 첫번째 노드
        /// </summary>
        public RTFNode FirstNode
        {
            get
            {
                if(this.childNodeList.Count > 0)
                {
                    return this.childNodeList[0];
                }
                else
                {
                    return null;
                }
            }
        }

        #endregion
        #region 키워드 - Keyword

        /// <summary>
        /// 키워드
        /// </summary>
        public override string Keyword
        {
            get
            {
                if(this.childNodeList.Count > 0)
                {
                    return this.childNodeList[0].Keyword;
                }
                else
                {
                    return null;
                }
            }
            set
            {
            }
        }

        #endregion
        #region 매개 변수 소유 여부 - HasParameter

        /// <summary>
        /// 매개 변수 소유 여부
        /// </summary>
        public override bool HasParameter
        {
            get
            {
                if(this.childNodeList.Count > 0)
                {
                    return this.childNodeList[0].HasParameter;
                }
                else
                {
                    return false;
                }
            }
            set
            {
            }
        }

        #endregion
        #region 매개 변수 - Parameter

        /// <summary>
        /// 매개 변수
        /// </summary>
        public override int Parameter
        {
            get
            {
                if(this.childNodeList.Count > 0)
                {
                    return this.childNodeList[0].Parameter;
                }
                else
                {
                    return 0;
                }
            }
        }

        #endregion
        #region 텍스트 - Text

        /// <summary>
        /// 텍스트
        /// </summary>
        public virtual string Text
        {
            get
            {
                StringBuilder stringBuilder = new StringBuilder();

                foreach(RTFNode childNode in this.childNodeList)
                {
                    if(childNode is RTFNodeGroup)
                    {
                        stringBuilder.Append(((RTFNodeGroup)childNode).Text);
                    }

                    if(childNode.NodeType == RTFNodeType.TEXT)
                    {
                        stringBuilder.Append(childNode.Keyword);
                    }
                }

                return stringBuilder.ToString();
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자  - RTFNodeGroup()

        /// <summary>
        /// 생성자
        /// </summary>
        public RTFNodeGroup()
        {
            this.nodeType = RTFNodeType.GROUP;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 자식 추가하기 - AppendChild(node)

        /// <summary>
        /// 자식 추가하기
        /// </summary>
        /// <param name="node">RTF 노드</param>
        public void AppendChild(RTFNode node)
        {
            CheckChildNodeList();

            if(node == null)
            {
                throw new ArgumentNullException("node");
            }

            if(node == this)
            {
                throw new ArgumentException("node != this");
            }

            node.Parent        = this;
            node.OwnerDocument = this.ownerDocument;

            ChildNodeList.Add(node);
        }

        #endregion
        #region 자식 삽입하기 - InsertChild(index, node)

        /// <summary>
        /// 자식 삽입하기
        /// </summary>
        /// <param name="index">인덱스</param>
        /// <param name="node">RTF 노드</param>
        public void InsertChild(int index, RTFNode node)
        {
            CheckChildNodeList();

            if(node == null)
            {
                throw new ArgumentNullException("node");
            }

            if(node == this)
            {
                throw new ArgumentException("node != this");
            }

            node.Parent        = this;
            node.OwnerDocument = this.ownerDocument;

            ChildNodeList.Insert(index, node);
        }

        #endregion
        #region 자식 제거하기 - RemoveChild(node)

        /// <summary>
        /// 자식 제거하기
        /// </summary>
        /// <param name="node">RTF 노드</param>
        public void RemoveChild(RTFNode node)
        {
            CheckChildNodeList();

            if(node == null)
            {
                throw new ArgumentNullException("node");
            }

            if(node == this)
            {
                throw new ArgumentException("node != this");
            }

            ChildNodeList.Remove(node);
        }

        #endregion
        #region 키 검색하기 - SearchKey(Keyword, searchChild)

        /// <summary>
        /// 키 검색하기
        /// </summary>
        /// <param name="Keyword">키워드</param>
        /// <param name="searchChild">자식 검색 여부</param>
        /// <returns>RTF 노드</returns>
        public RTFNode SearchKey(string Keyword, bool searchChild)
        {
            foreach(RTFNode node in this.childNodeList)
            {
                if(node.NodeType == RTFNodeType.KEYWORD || node.NodeType == RTFNodeType.EXTERNAL_KEYWORD || node.NodeType == RTFNodeType.CONTROL)
                {
                    if(node.Keyword == Keyword)
                    {
                        return node;
                    }
                }

                if(searchChild)
                {
                    if(node is RTFNodeGroup)
                    {
                        RTFNodeGroup nodeGroup = (RTFNodeGroup)node;

                        RTFNode nodeFound = nodeGroup.SearchKey(Keyword, true);

                        if(nodeFound != null)
                        {
                            return nodeFound;
                        }
                    }
                }
            }

            return null;
        }

        #endregion
        #region 노드 리스트 구하기 - GetNodeList(includeGroupNode)

        /// <summary>
        /// 노드 리스트 구하기
        /// </summary>
        /// <param name="includeGroupNode">그룹 노드 포함 여부</param>
        /// <returns>노드 리스트</returns>
        public RTFNodeList GetNodeList(bool includeGroupNode)
        {
            RTFNodeList list = new RTFNodeList();

            AddAllNodes(list, includeGroupNode);

            return list ;
        }

        #endregion
        #region 쓰기 - Write(writer)

        /// <summary>
        /// 쓰기
        /// </summary>
        /// <param name="writer">RTF 라이터</param>
        public override void Write(RTFWriter writer)
        {
            writer.WriteStartGroup();

            foreach(RTFNode childNode in this.childNodeList)
            {
                childNode.Write(writer);
            }

            writer.WriteEndGroup();
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Internal

        #region 텍스트 병합하기 - MergeText()

        /// <summary>
        /// 텍스트 병합하기
        /// </summary>
        internal void MergeText()
        {
            RTFNodeList list = new RTFNodeList();

            StringBuilder stringBuilder = new StringBuilder();

            ByteBuffer buffer = new ByteBuffer();

            foreach(RTFNode childNode in this.childNodeList)
            {
                if(childNode.NodeType == RTFNodeType.TEXT)
                {
                    AddString(stringBuilder, buffer);

                    stringBuilder.Append(childNode.Keyword);

                    continue ;
                }

                if(childNode.NodeType == RTFNodeType.CONTROL && childNode.Keyword == "\'" && childNode.HasParameter)
                {
                    buffer.Add((byte)childNode.Parameter);

                    continue ;
                }
                else if(childNode.NodeType == RTFNodeType.CONTROL || childNode.NodeType == RTFNodeType.KEYWORD)
                {
                    if(childNode.Keyword == "tab")
                    {
                        AddString(stringBuilder, buffer);

                        stringBuilder.Append('\t');

                        continue;
                    }

                    if(childNode.Keyword == "emdash")
                    {
                        AddString(stringBuilder, buffer);

                        stringBuilder.Append('j');

                        continue;
                    }

                    if(childNode.Keyword == string.Empty)
                    {
                        AddString(stringBuilder, buffer);

                        stringBuilder.Append('Ƀ');

                        continue;
                    }
                }

                AddString(stringBuilder, buffer);

                if(stringBuilder.Length > 0)
                {
                    list.Add(new RTFNode(RTFNodeType.TEXT, stringBuilder.ToString()));

                    stringBuilder = new StringBuilder();
                }

                list.Add(childNode);
            }

            AddString(stringBuilder, buffer);

            if(stringBuilder.Length > 0)
            {
                list.Add(new RTFNode(RTFNodeType.TEXT, stringBuilder.ToString()));
            }

            this.childNodeList.Clear();

            foreach(RTFNode node in list)
            {
                node.Parent        = this;
                node.OwnerDocument = this.ownerDocument;

                this.childNodeList.Add(node);
            }
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 모든 노드 추가하기 - AddAllNodes(targetNodeList, includeGroupNode)

        /// <summary>
        /// 모든 노드 추가하기
        /// </summary>
        /// <param name="targetNodeList">타겟 노드 리스트</param>
        /// <param name="includeGroupNode">그룹 노드 포함 여부</param>
        private void AddAllNodes(RTFNodeList targetNodeList, bool includeGroupNode)
        {
            foreach(RTFNode childNode in this.childNodeList)
            {
                if(childNode is RTFNodeGroup)
                {
                    if(includeGroupNode)
                    {
                        targetNodeList.Add(childNode);
                    }

                    ((RTFNodeGroup)childNode).AddAllNodes(targetNodeList, includeGroupNode);
                }
                else
                {
                    targetNodeList.Add(childNode);
                }
            }
        }

        #endregion
        #region 문자열 추가하기 - AddString(stringBuilder, buffer)

        /// <summary>
        /// 문자열 추가하기
        /// </summary>
        /// <param name="stringBuilder">문자열 빌더</param>
        /// <param name="buffer">버퍼</param>
        private void AddString(StringBuilder stringBuilder, ByteBuffer buffer)
        {
            if(buffer.Count > 0)
            {
                string text = buffer.GetString(this.ownerDocument.RuntimeEncoding);

                stringBuilder.Append(text);

                buffer.Reset();
            }
        }

        #endregion
        #region 자식 노드 리스트 체크하기 - CheckChildNodeList()

        /// <summary>
        /// 자식 노드 리스트 체크하기
        /// </summary>
        private void CheckChildNodeList()
        {
            if(ChildNodeList == null)
            {
                throw new Exception("child node is invalidate");
            }
        }

        #endregion
    }
}