using System;
using System.Collections;
using System.Drawing;
using System.IO;
using System.Text;

namespace TestLibrary
{
    /// <summary>
    /// RTF  
    /// </summary>
    public class RTFRawDocument : RTFNodeGroup
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Protected

        #region Field

        /// <summary>
        ///  
        /// </summary>
        protected RTFDocumentInfo documentInfo = new RTFDocumentInfo();

        /// <summary>
        ///  ̺
        /// </summary>
        protected RTFColorTable colorTable = new RTFColorTable();

        /// <summary>
        /// Ʈ ̺
        /// </summary>
        protected RTFFontTable fontTable = new RTFFontTable();

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// ڵ
        /// </summary>
        private Encoding encoding = null;

        /// <summary>
        /// Ʈ  Ʈ ڵ
        /// </summary>
        private Encoding fontChartSetEncoding = null;

        /// <summary>
        ///  Ʈ  Ʈ ڵ
        /// </summary>
        private Encoding associateFontChartSetEncoding = null;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region   - DocumentInfo

        /// <summary>
        ///  
        /// </summary>
        public RTFDocumentInfo DocumentInfo
        {
            get
            {
                return this.documentInfo;
            }
        }

        #endregion
        #region θ - Parent

        /// <summary>
        /// θ
        /// </summary>
        public override RTFNodeGroup Parent
        {
            get
            {
                return null;
            }
            set
            {
            }
        }

        #endregion
        #region   - OwnerDocument

        /// <summary>
        ///  
        /// </summary>
        public override RTFRawDocument OwnerDocument
        {
            get
            {
                return this;
            }
            set
            {
            }
        }

        #endregion
        #region  ̺ - ColorTable

        /// <summary>
        ///  ̺
        /// </summary>
        public RTFColorTable ColorTable
        {
            get
            {
                return this.colorTable;
            }
        }

        #endregion
        #region Ʈ ̺ - FontTable

        /// <summary>
        /// Ʈ ̺
        /// </summary>
        public RTFFontTable FontTable
        {
            get
            {
                return this.fontTable;
            }
        }

        #endregion
        #region ڵ - Encoding

        /// <summary>
        /// ڵ
        /// </summary>
        public Encoding Encoding
        {
            get
            {
                if(this.encoding == null)
                {
                    RTFNode node = childNodeList[RTFConstant.ANSICPG];

                    if(node != null && node.HasParameter)
                    {
                        this.encoding = Encoding.GetEncoding(node.Parameter);
                    }
                }

                if(this.encoding == null)
                {
                    this.encoding = Encoding.Default;
                }

                return this.encoding;
            }
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Internal

        #region  ڵ - RuntimeEncoding

        /// <summary>
        ///  ڵ
        /// </summary>
        internal Encoding RuntimeEncoding
        {
            get
            {
                if(this.fontChartSetEncoding != null)
                {
                    return this.fontChartSetEncoding;
                }

                if(this.associateFontChartSetEncoding != null)
                {
                    return this.associateFontChartSetEncoding;
                }

                return this.Encoding;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region  - RTFRawDocument()

        /// <summary>
        /// 
        /// </summary>
        public RTFRawDocument()
        {
            this.ownerDocument = this;
            this.parent        = null;

            this.colorTable.CheckValueExistWhenAdd = false;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region εϱ - Load(reader)

        /// <summary>
        /// εϱ
        /// </summary>
        /// <param name="reader">RTF </param>
        public void Load(RTFReader reader)
        {
            this.childNodeList.Clear();

            Stack stack = new Stack();

            RTFNodeGroup newNodeGroup = null;
            RTFNode      newNode      = null;

            while(reader.ReadToken() != null)
            {
                if(reader.CurrentTokenType == RTFTokenType.GroupStart)
                {
                    if(newNodeGroup == null)
                    {
                        newNodeGroup = this;
                    }
                    else
                    {
                        newNodeGroup = new RTFNodeGroup();

                        newNodeGroup.OwnerDocument = this;
                    }

                    if(newNodeGroup != this)
                    {
                        RTFNodeGroup peekNodeGroup = (RTFNodeGroup)stack.Peek();

                        peekNodeGroup.AppendChild(newNodeGroup);
                    }

                    stack.Push(newNodeGroup);
                }
                else if(reader.CurrentTokenType == RTFTokenType.GroupEnd)
                {
                    newNodeGroup = (RTFNodeGroup)stack.Pop();

                    newNodeGroup.MergeText();

                    if(newNodeGroup.FirstNode is RTFNode)
                    {
                        switch(newNodeGroup.Keyword)
                        {
                            case RTFConstant.FONTTBL :

                                ReadFontTable(newNodeGroup);

                                break;

                            case RTFConstant.COLORTBL :

                                ReadColorTable(newNodeGroup);

                                break;

                            case RTFConstant.INFO :

                                ReadDocumentInfo(newNodeGroup);

                                break;
                        }
                    }

                    if(stack.Count > 0)
                    {
                        newNodeGroup = (RTFNodeGroup)stack.Peek();
                    }
                    else
                    {
                        break;
                    }
                }
                else
                {
                    newNode = new RTFNode(reader.CurrentToken);

                    newNode.OwnerDocument = this;

                    newNodeGroup.AppendChild(newNode);

                    if(newNode.Keyword == RTFConstant.F)
                    {
                        RTFFont font = this.FontTable[newNode.Parameter];

                        if(font != null)
                        {
                            this.fontChartSetEncoding = font.Encoding;
                        }
                        else
                        {
                            this.fontChartSetEncoding = null;
                        }
                    }
                    else if(newNode.Keyword == RTFConstant.AF)
                    {
                        RTFFont font = this.FontTable[newNode.Parameter];

                        if(font != null)
                        {
                            this.associateFontChartSetEncoding = font.Encoding;
                        }
                        else
                        {
                            this.associateFontChartSetEncoding = null;
                        }
                    }
                }
            }
            while(stack.Count > 0)
            {
                newNodeGroup = (RTFNodeGroup)stack.Pop();

                newNodeGroup.MergeText();
            }
        }

        #endregion
        #region εϱ- Load(filePath)

        /// <summary>
        /// εϱ
        /// </summary>
        /// <param name="filePath"> </param>
        public void Load(string filePath)
        {
            this.encoding = null;

            using(RTFReader reader = new RTFReader())
            {
                if(reader.LoadRTFFile(filePath))
                {
                    Load(reader);

                    reader.Close();
                }

                reader.Close();
            }
        }

        #endregion
        #region εϱ - Load(textReader)

        /// <summary>
        /// εϱ
        /// </summary>
        /// <param name="textReader">ؽƮ </param>
        public void Load(TextReader textReader)
        {
            RTFReader rtfReader = new RTFReader();

            rtfReader.LoadReader(textReader);

            Load(rtfReader);
        }

        #endregion
        #region RTF ؽƮ εϱ - LoadRTFText(rtf)

        /// <summary>
        /// RTF ؽƮ εϱ
        /// </summary>
        /// <param name="rtf">RTF</param>
        public void LoadRTFText(string rtf)
        {
            this.encoding = null;

            using(RTFReader reader = new RTFReader())
            {
                if(reader.LoadRTFText(rtf))
                {
                    Load(reader);

                    reader.Close();
                }

                reader.Close();
            }
        }

        #endregion
        #region  - Write(writer)

        /// <summary>
        /// 
        /// </summary>
        /// <param name="writer">RTF </param>
        public override void Write(RTFWriter writer)
        {
            writer.Encoding = this.Encoding;

            base.Write(writer);
        }

        #endregion
        #region ϱ - Save(filePath)

        /// <summary>
        /// ϱ
        /// </summary>
        /// <param name="filePath"> </param>
        public void Save(string filePath)
        {
            using(RTFWriter writer = new RTFWriter(filePath))
            {
                Write(writer);

                writer.Close();
            }
        }

        #endregion
        #region ϱ - Save(stream)

        /// <summary>
        /// ϱ
        /// </summary>
        /// <param name="stream">Ʈ</param>
        public void Save(Stream stream)
        {
            using(RTFWriter writer = new RTFWriter(new StreamWriter(stream , Encoding)))
            {
                Write(writer);

                writer.Close();
            }
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Ʈ ̺ б - ReadFontTable(nodeGroup)

        /// <summary>
        /// Ʈ ̺ б
        /// </summary>
        /// <param name="nodeGroup"> ׷</param>
        private void ReadFontTable(RTFNodeGroup nodeGroup)
        {
            this.fontTable.Clear();

            foreach(RTFNode node in nodeGroup.ChildNodeList)
            {
                if(node is RTFNodeGroup)
                {
                    int index = -1;

                    string name         = null;
                    int    characterSet = 0;

                    foreach(RTFNode item in node.ChildNodeList)
                    {
                        if(item.Keyword == "f" && item.HasParameter)
                        {
                            index = item.Parameter;
                        }
                        else if(item.Keyword == RTFConstant.FCHARSET)
                        {
                            characterSet = item.Parameter;
                        }
                        else if(item.NodeType == RTFNodeType.TEXT)
                        {
                            if(item.Keyword != null && item.Keyword.Length > 0)
                            {
                                name = item.Keyword;

                                break;
                            }
                        }
                    }

                    if(index >= 0 && name != null)
                    {
                        if(name.EndsWith(";"))
                        {
                            name = name.Substring(0, name.Length - 1);
                        }

                        name = name.Trim();

                        RTFFont font = new RTFFont(index, name);

                        font.CharacterSet = characterSet;

                        this.fontTable.Add(font);
                    }
                }
            }
        }

        #endregion
        #region  ̺ б - ReadColorTable(nodeGroup)

        /// <summary>
        ///  ̺ б
        /// </summary>
        /// <param name="nodeGroup"> ׷</param>
        private void ReadColorTable(RTFNodeGroup nodeGroup)
        {
            this.colorTable.Clear();

            int red   = -1;
            int green = -1;
            int blue  = -1;

            foreach(RTFNode node in nodeGroup.ChildNodeList)
            {
                if(node.Keyword == "red")
                {
                    red = node.Parameter;
                }
                else if(node.Keyword == "green")
                {
                    green = node.Parameter;
                }
                else if(node.Keyword == "blue")
                {
                    blue = node.Parameter;
                }

                if(node.Keyword == ";")
                {
                    if(red >= 0 && green >= 0 && blue >= 0)
                    {
                        Color color = Color.FromArgb(255, red, green, blue);

                        this.colorTable.Add(color);

                        red   = -1;
                        green = -1;
                        blue  = -1;
                    }
                }
            }

            if(red >= 0 && green >= 0 && blue >= 0)
            {
                Color color = Color.FromArgb(255, red, green, blue);

                this.colorTable.Add(color);
            }
        }

        #endregion
        #region   б - ReadDocumentInfo(nodeGroup)

        /// <summary>
        ///   б
        /// </summary>
        /// <param name="nodeGroup"> ׷</param>
        private void ReadDocumentInfo(RTFNodeGroup nodeGroup)
        {
            this.documentInfo.Clear();

            foreach(RTFNode node in nodeGroup.ChildNodeList)
            {
                if((node is RTFNodeGroup) == false)
                {
                    continue;
                }

                if(node.Keyword == "creatim")
                {
                    this.documentInfo.CreateTime = ReadDateTime(node);
                }
                else if(node.Keyword == "revtim")
                {
                    this.documentInfo.UpdateTime = ReadDateTime(node);
                }
                else if(node.Keyword == "printim")
                {
                    this.documentInfo.PrintTime = ReadDateTime(node);
                }
                else if(node.Keyword == "buptim")
                {
                    this.documentInfo.BackupTime = ReadDateTime(node);
                }
                else
                {
                    if(node.HasParameter)
                    {
                        this.documentInfo.SetInfo(node.Keyword, node.Parameter.ToString());
                    }
                    else
                    {
                        this.documentInfo.SetInfo(node.Keyword, node.ChildNodeList.Text);
                    }
                }
            }
        }

        #endregion
        #region ¥/ð б - ReadDateTime(node)

        /// <summary>
        /// ¥/ð б
        /// </summary>
        /// <param name="node"></param>
        /// <returns>¥/ð</returns>
        private DateTime ReadDateTime(RTFNode node)
        {
            int year   = node.ChildNodeList.GetParameter("yr" , 1900);
            int month  = node.ChildNodeList.GetParameter("mo" , 1  );
            int day    = node.ChildNodeList.GetParameter("dy" , 1  );
            int hour   = node.ChildNodeList.GetParameter("hr" , 0  );
            int minute = node.ChildNodeList.GetParameter("min", 0  );
            int second = node.ChildNodeList.GetParameter("sec", 0  );

            return new DateTime(year, month, day, hour, minute, second);
        }

        #endregion
    }
}