﻿using System.Text;

namespace TestLibrary
{
    /// <summary>
    /// RTF 텍스트 컨테이너
    /// </summary>
    internal class RTFTextContainer
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 문서
        /// </summary>
        private RTFDOMDocument document = null;

        /// <summary>
        /// 버퍼
        /// </summary>
        private ByteBuffer buffer = new ByteBuffer();

        /// <summary>
        /// 문자열 빌더
        /// </summary>
        private StringBuilder stringBuilder = new StringBuilder();

        /// <summary>
        /// 레벨
        /// </summary>
        private int level = 0;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 문서 - Document

        /// <summary>
        /// 문서
        /// </summary>
        public RTFDOMDocument Document
        {
            get
            {
                return this.document;
            }
            set
            {
                this.document = value;
            }
        }

        #endregion
        #region 컨텐트 소유 여부 - HasContent

        /// <summary>
        /// 컨텐트 소유 여부
        /// </summary>
        public bool HasContent
        {
            get
            {
                CheckBuffer();

                return this.stringBuilder.Length > 0;
            }
        }

        #endregion
        #region 텍스트 - Text

        /// <summary>
        /// 텍스트
        /// </summary>
        public string Text
        {
            get
            {
                CheckBuffer();

                return this.stringBuilder.ToString();
            }
        }

        #endregion
        #region 레벨 - Level

        /// <summary>
        /// 레벨
        /// </summary>
        public int Level
        {
            get
            {
                return this.level;
            }
            set
            {
                this.level = value;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - RTFTextContainer(document)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="document">문서</param>
        public RTFTextContainer(RTFDOMDocument document)
        {
            this.document = document;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 추가하기 - Append(text)

        /// <summary>
        /// 추가하기
        /// </summary>
        /// <param name="text">텍스트</param>
        public void Append(string text)
        {
            if(string.IsNullOrEmpty(text) == false)
            {
                CheckBuffer();

                this.stringBuilder.Append(text);
            }
        }

        #endregion
        #region 수락하기 - Accept(token, reader)

        /// <summary>
        /// 수락하기
        /// </summary>
        /// <param name="token">토큰</param>
        /// <param name="reader">리더</param>
        /// <returns>처리 결과</returns>
        public bool Accept(RTFToken token, RTFReader reader)
        {
            if(token == null)
            {
                return false;
            }

            if(token.TokenType == RTFTokenType.Text)
            {
                if(reader != null)
                {
                    if(token.Keyword[0] == '?')
                    {
                        if(reader.LastToken != null)
                        {
                            if(reader.LastToken.TokenType == RTFTokenType.Keyword && reader.LastToken.Keyword == "u" && reader.LastToken.HasParameter)
                            {
                                if(token.Keyword.Length > 0)
                                {
                                    CheckBuffer();
                                }

                                return true;
                            }
                        }
                    }
                }

                CheckBuffer();

                this.stringBuilder.Append(token.Keyword);

                return true ;
            }
            else if(token.TokenType == RTFTokenType.Control && token.Keyword == "'" && token.HasParameter)
            {
                if(reader.CurrentLayerInfo.CheckUCValueCount())
                {
                    this.buffer.Add((byte)token.Parameter);
                }

                return true;
            }

            if(token.Keyword == RTFConstant.U && token.HasParameter)
            {
                CheckBuffer();

                this.stringBuilder.Append((char)token.Parameter);

                reader.CurrentLayerInfo.UCValueCount = reader.CurrentLayerInfo.UCValue;

                return true;
            }

            if(token.Keyword == "tab")
            {
                CheckBuffer();

                this.stringBuilder.Append("\t");

                return true;
            }

            if(token.Keyword == "emdash")
            {
                CheckBuffer();

                this.stringBuilder.Append('—');

                return true;
            }

            if(token.Keyword == "")
            {
                CheckBuffer();

                this.stringBuilder.Append('-');

                return true;
            }

            return false;
        }

        #endregion
        #region 지우기 - Clear()

        /// <summary>
        /// 지우기
        /// </summary>
        public void Clear()
        {
            this.buffer.Clear();

            this.stringBuilder = new StringBuilder();
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 버퍼 체크하기 - CheckBuffer()

        /// <summary>
        /// 버퍼 체크하기
        /// </summary>
        private void CheckBuffer()
        {
            if(this.buffer.Count > 0)
            {
                string text = this.buffer.GetString(this.document.RuntimeEncoding);

                this.stringBuilder.Append(text);

                this.buffer.Clear();
            }
        }

        #endregion
    }
}