﻿namespace TestLibrary
{
    /// <summary>
    /// rtf token type
    /// </summary>
    public class RTFToken
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 토큰 타입
        /// </summary>
        private RTFTokenType tokenType = RTFTokenType.None;

        /// <summary>
        /// 키워드
        /// </summary>
        private string keyword = null;

        /// <summary>
        /// 부모
        /// </summary>
        private RTFToken parent = null;

        /// <summary>
        /// 매개 변수 소유 여부
        /// </summary>
        private bool hasParameter = false;

        /// <summary>
        /// 매개 변수
        /// </summary>
        private int parameter = 0;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 토큰 타입 - TokenType

        /// <summary>
        /// 토큰 타입
        /// </summary>
        public RTFTokenType TokenType
        {
            get
            {
                return this.tokenType;
            }
            set
            {
                this.tokenType = value;
            }
        }

        #endregion
        #region 키워드 - Keyword

        /// <summary>
        /// 키워드
        /// </summary>
        public string Keyword
        {
            get
            {
                return this.keyword;
            }
            set
            {
                this.keyword = value;
            }
        }

        #endregion
        #region 부모 - Parent

        /// <summary>
        /// 부모
        /// </summary>
        public RTFToken Parent
        {
            get
            {
                return this.parent;
            }
            set
            {
                this.parent = value;
            }
        }

        #endregion
        #region 매개 변수 소유 여부 - HasParameter

        /// <summary>
        /// 매개 변수 소유 여부
        /// </summary>
        public bool HasParameter
        {
            get
            {
                return this.hasParameter;
            }
            set
            {
                this.hasParameter = value;
            }
        }

        #endregion
        #region 매개 변수 - Parameter

        /// <summary>
        /// 매개 변수
        /// </summary>
        public int Parameter
        {
            get
            {
                return this.parameter;
            }
            set
            {
                this.parameter = value;
            }
        }

        #endregion
        #region 텍스트 토큰 여부 - IsTextToken

        /// <summary>
        /// 텍스트 토큰 여부
        /// </summary>
        public bool IsTextToken
        {
            get
            {
                if(this.tokenType == RTFTokenType.Text)
                {
                    return true;
                }

                if(this.tokenType == RTFTokenType.Control && this.keyword == "'" && this.hasParameter)
                {
                    return true;
                }

                return false;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 문자열 구하기 - ToString()

        /// <summary>
        /// 문자열 구하기
        /// </summary>
        /// <returns>문자열</returns>
        public override string ToString()
        {
            if(this.tokenType == RTFTokenType.Keyword)
            {
                return Keyword + Parameter;
            }
            else if(this.tokenType == RTFTokenType.GroupStart)
            {
                return "{";
            }
            else if(this.tokenType == RTFTokenType.GroupEnd)
            {
                return "}";
            }
            else if(this.tokenType == RTFTokenType.Text)
            {
                return "Text:" + Parameter;
            }

            return this.tokenType.ToString() + ":" + Keyword + " " + Parameter;
        }

        #endregion
    }
}