using System;
using System.IO;
using System.Text;

namespace TestLibrary
{
    /// <summary>
    /// RTF 
    /// </summary>
    public class RTFWriter : IDisposable
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// ؽƮ 
        /// </summary>
        private TextWriter writer = null;

        /// <summary>
        /// ڵ
        /// </summary>
        private Encoding encoding = Encoding.Default ;

        /// <summary>
        /// 鿩 
        /// </summary>
        private bool indent = false;

        /// <summary>
        /// 鿩 ڿ
        /// </summary>
        private string indentString = "   ";

        /// <summary>
        /// ׷ 
        /// </summary>
        private int groupLevel = 0;

        /// <summary>
        ///  ġ
        /// </summary>
        private int currentPosition = 0 ;

        /// <summary>
        ///   
        /// </summary>
        private int currentLineHead = 0 ;

        /// <summary>
        /// 16 ڿ
        /// </summary>
        private const string HEXADECIMAL = "0123456789abcdef";

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region ڵ - Encoding

        /// <summary>
        /// ڵ
        /// </summary>
        public Encoding Encoding
        {
            get
            {
                return this.encoding;
            }
            set
            {
                this.encoding = value;
            }
        }

        #endregion
        #region 鿩 - Indent

        /// <summary>
        /// 鿩
        /// </summary>
        public bool Indent
        {
            get
            {
                return this.indent;
            }
            set
            {
                this.indent = value;
            }
        }

        #endregion
        #region 鿩 ڿ - IndentString

        /// <summary>
        /// 鿩 ڿ
        /// </summary>
        public string IndentString
        {
            get
            {
                return this.indentString;
            }
            set
            {
                this.indentString = value;
            }
        }

        #endregion
        #region ׷  - GroupLevel

        /// <summary>
        /// ׷ 
        /// </summary>
        public int GroupLevel
        {
            get
            {
                return this.groupLevel;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region  - RTFWriter(writer)

        /// <summary>
        /// 
        /// </summary>
        /// <param name="writer">ؽƮ </param>
        public RTFWriter(TextWriter writer)
        {
            this.writer = writer;
        }

        #endregion
        #region  - RTFWriter(filePath)

        /// <summary>
        /// 
        /// </summary>
        /// <param name="filePath"> </param>
        public RTFWriter(string filePath)
        {
            this.writer = new StreamWriter(filePath, false, Encoding.ASCII);
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region ݱ - Close()

        /// <summary>
        /// ݱ
        /// </summary>
        public void Close()
        {
            if(this.groupLevel > 0)
            {
                throw new Exception("Some group does not finish");
            }

            if(this.writer != null)
            {
                this.writer.Close();

                this.writer = null;
            }
        }

        #endregion
        #region   - Flush()

        /// <summary>
        ///  
        /// </summary>
        public void Flush()
        {
            if(this.writer != null)
            {
                this.writer.Flush();
            }
        }

        #endregion
        #region ׷  - WriteGroup(keyword)

        /// <summary>
        /// ׷ 
        /// </summary>
        /// <param name="keyword">Ű</param>
        public void WriteGroup(string keyword)
        {
            this.WriteStartGroup();

            this.WriteKeyword(keyword);

            this.WriteEndGroup();
        }

        #endregion
        #region ׷   - WriteStartGroup()

        /// <summary>
        /// ׷  
        /// </summary>
        public void WriteStartGroup()
        {
            if(this.indent)
            {
                WriteNewLineInternal();

                this.writer.Write("{");
            }
            else
            {
                this.writer.Write("{");
            }

            this.groupLevel++;
        }

        #endregion
        #region ׷   - WriteEndGroup()

        /// <summary>
        /// ׷  
        /// </summary>
        public void WriteEndGroup()
        {
            this.groupLevel--;

            if(this.groupLevel < 0)
            {
                throw new Exception("group level error");
            }

            if(this.indent)
            {
                WriteNewLineInternal();

                WriteInternal("}");
            }
            else
            {
                WriteInternal("}");
            }
        }

        #endregion
        #region   - WriteRaw(text)

        /// <summary>
        ///  
        /// </summary>
        /// <param name="text">ؽƮ</param>
        public void WriteRaw(string text)
        {
            if(text != null && text.Length > 0)
            {
                WriteInternal(text);
            }
        }

        #endregion
        #region Ű  - WriteKeyword(keyword , externKeyword)

        /// <summary>
        /// Ű 
        /// </summary>
        /// <param name="keyword">Ű</param>
        /// <param name="externKeyword">ܺ Ű </param>
        public void WriteKeyword(string keyword , bool externKeyword)
        {
            if(keyword == null || keyword.Length == 0)
            {
                throw new ArgumentNullException("keyword");
            }

            if(this.indent == false && (keyword == "par" || keyword == "pard"))
            {
                WriteInternal(Environment.NewLine);
            }

            if(this.indent)
            {
                if(keyword == "par" || keyword == "pard")
                {
                    WriteNewLineInternal();
                }
            }

            if(externKeyword)
            {
                WriteInternal("\\*\\");
            }
            else
            {
                WriteInternal("\\");
            }

            WriteInternal(keyword);
        }

        #endregion
        #region Ű  - WriteKeyword(keyword)

        /// <summary>
        /// Ű 
        /// </summary>
        /// <param name="keyword">Ű</param>
        public void WriteKeyword(string keyword)
        {
            WriteKeyword(keyword, false);
        }

        #endregion
        #region ؽƮ  - WriteText(text , addWhitespace)

        /// <summary>
        /// ؽƮ 
        /// </summary>
        /// <param name="text">ؽƮ</param>
        /// <param name="addWhitespace">  ߰ </param>
        public void WriteText(string text , bool addWhitespace)
        {
            if(text == null || text.Length == 0)
            {
                return;
            }
            
            if(addWhitespace)
            {
                WriteInternal(' ');
            }

            for(int i = 0; i < text.Length; i ++)
            {
                char character = text[i];

                WriteCharacterInternal(character);
            }
        }

        #endregion
        #region ؽƮ  - WriteText(text)

        /// <summary>
        /// ؽƮ 
        /// </summary>
        /// <param name="text">ؽƮ</param>
        public void WriteText(string text)
        {
            if(text == null || text.Length == 0)
            {
                return ;
            }

            WriteText(text , true);
        }

        #endregion
        #region ڵ ؽƮ  - WriteUnicodeText(text)

        /// <summary>
        /// ڵ ؽƮ 
        /// </summary>
        /// <param name="text">ؽƮ</param>
        public void WriteUnicodeText(string text)
        {
            if(string.IsNullOrEmpty(text) == false)
            {
                WriteKeyword("uc1");

                foreach(char character in text)
                {
                    if(character > 127)
                    {
                        int   value1 = (int)character;
                        short value2 = (short)value1;

                        WriteKeyword("u" + value2.ToString());

                        WriteRaw(" ?");
                    }
                    else
                    {
                        WriteCharacterInternal(character);
                    }
                }
            }
        }

        #endregion
        #region Ʈ  - WriteByte(value)

        /// <summary>
        /// Ʈ 
        /// </summary>
        /// <param name="value"></param>
        public void WriteByte(byte value)
        {
            int high = (value & 0xf0) >> 4;
            int low  = value & 0xf;

            this.writer.Write(HEXADECIMAL[high]);
            this.writer.Write(HEXADECIMAL[low ]);

            this.currentPosition += 2 ;
        }

        #endregion
        #region Ʈ 迭  - WriteByteArray(byteArray)

        /// <summary>
        /// Ʈ 迭 
        /// </summary>
        /// <param name="byteArray">Ʈ 迭</param>
        public void WriteByteArray(byte[] byteArray)
        {
            if(byteArray == null || byteArray.Length == 0)
            {
                return;
            }

            WriteRaw(" ");

            for(int i = 0; i < byteArray.Length; i++)
            {
                if((i % 32) == 0)
                {
                    WriteRaw(Environment.NewLine);

                    WriteInternal();
                }
                else if((i % 8) == 0)
                {
                    WriteRaw(" ");
                }

                byte value = byteArray[i];

                int high = (value & 0xf0) >> 4;
                int low  = value & 0xf;

                this.writer.Write(HEXADECIMAL[high]);
                this.writer.Write(HEXADECIMAL[low ]);

                this.currentPosition += 2;
            }
        }

        #endregion
        #region ҽ ϱ - Dispose()

        /// <summary>
        /// ҽ ϱ
        /// </summary>
        public void Dispose()
        {
            Close();
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region   (ο) - WriteCharacterInternal(character)

        /// <summary>
        ///   (ο)
        /// </summary>
        /// <param name="character"></param>
        private void WriteCharacterInternal(char character)
        {
            if(character == '\t')
            {
                WriteKeyword("tab");

                WriteInternal(' ');
            }

            if(character > 32 && character < 127)
            {
                if(character == '\\' || character == '{' || character == '}')
                {
                    WriteInternal('\\');
                }

                WriteInternal(character);
            }
            else
            {
                byte[] byteArray = this.encoding.GetBytes(character.ToString());

                for(int i = 0; i < byteArray.Length; i++)
                {
                    WriteInternal("\\\'");

                    WriteByte(byteArray[i]);
                }
            }
        }

        #endregion
        #region  (ο) - WriteInternal(character)

        /// <summary>
        ///  (ο)
        /// </summary>
        /// <param name="character"></param>
        private void WriteInternal(char character)
        {
            this.currentPosition++;

            this.writer.Write(character);
        }

        #endregion
        #region  (ο) - WriteInternal(text)

        /// <summary>
        ///  (ο)
        /// </summary>
        /// <param name="text">ؽƮ</param>
        private void WriteInternal(string text)
        {
            this.currentPosition += text.Length;

            this.writer.Write(text);
        }

        #endregion
        #region  (ο) - WriteInternal()

        /// <summary>
        ///  (ο)
        /// </summary>
        private void WriteInternal()
        {
            if(this.indent)
            {
                for(int i = 0; i < this.groupLevel; i++)
                {
                    WriteInternal(this.indentString);
                }
            }
        }

        #endregion
        #region    (ο) - WriteNewLineInternal()

        /// <summary>
        ///    (ο)
        /// </summary>
        private void WriteNewLineInternal()
        {
            if(this.indent)
            {
                if(this.currentPosition > 0)
                {
                    WriteInternal(Environment.NewLine);

                    this.currentLineHead = this.currentPosition;

                    WriteInternal();
                }
            }
        }

        #endregion
    }
}