﻿using System;
using System.Collections;
using System.Diagnostics;

namespace TestLibrary
{
    /// <summary>
    /// 문자열 특성 컬렉션
    /// </summary>
    [Serializable]
    [DebuggerTypeProxy(typeof(RTFInstanceDebugView))]
    public class StringAttributeCollection : CollectionBase
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 인덱서 - this[name]

        /// <summary>
        /// 인덱서
        /// </summary>
        /// <param name="name">명칭</param>
        /// <returns>값</returns>
        public string this[string name]
        {
            get
            {
                foreach(StringAttribute attribute in this)
                {
                    if(attribute.Name == name)
                    {
                        return attribute.Value;
                    }
                }

                return null;
            }
            set
            {
                foreach(StringAttribute attribute in this)
                {
                    if(attribute.Name == name)
                    {
                        if(value == null)
                        {
                            List.Remove(attribute);
                        }
                        else
                        {
                            attribute.Value = value;
                        }

                        return;
                    }
                }

                if(value != null)
                {
                    StringAttribute newAttribute = new StringAttribute();

                    newAttribute.Name  = name;
                    newAttribute.Value = value;

                    List.Add(newAttribute);
                }
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - StringAttributeCollection()

        /// <summary>
        /// 생성자
        /// </summary>
        public StringAttributeCollection()
        {
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 추가하기 - Add(item)

        /// <summary>
        /// 추가하기
        /// </summary>
        /// <param name="item">항목</param>
        /// <returns>인덱스</returns>
        public int Add(StringAttribute item)
        {
            return List.Add(item);
        }

        #endregion
        #region 제거하기 - Remove(item)

        /// <summary>
        /// 제거하기
        /// </summary>
        /// <param name="item">항목</param>
        public void Remove(StringAttribute item)
        {
            List.Remove(item);
        }

        #endregion
    }
}