﻿using System;
using System.IO;
using System.Text;
using System.Windows.Forms;

using TestLibrary;

namespace TestProject
{
    /// <summary>
    /// 메인 폼
    /// </summary>
    public partial class MainForm : Form
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainForm()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainForm()
        {
            InitializeComponent();

            this.loadRTFFileToolStripButton1.Click       += loadRTFFileToolStripButton1_Click;
            this.pasteFromClipboardToolStripButton.Click += pasteFromClipboardToolStripButton_Click;
            this.loadRTFFileToolStripButton2.Click       += loadRTFFileToolStripButton2_Click;
            this.copyTextFormatToolStripButton.Click     += copyTextFormatToolStripButton_Click;
            this.copyRTFFormatToolStripButton.Click      += copyRTFFormatToolStripButton_Click;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region RTF DOM 문서 진행시 처리하기 - document_Progress(sender, e)

        /// <summary>
        /// RTF DOM 문서 진행시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void document_Progress(object sender, ProgressEventArgs e)
        {
            this.toolStripProgressBar.Maximum = e.MaximumValue;
            this.toolStripProgressBar.Value   = e.Value;
        }

        #endregion

        #region RTF 파일 로드 툴 스트립 버튼 1 클릭시 처리하기 - loadRTFFileToolStripButton1_Click(sender, e)

        /// <summary>
        /// RTF 파일 로드 툴 스트립 버튼 1 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void loadRTFFileToolStripButton1_Click(object sender, EventArgs e)
        {
            using(OpenFileDialog openFileDialog = new OpenFileDialog())
            {
                openFileDialog.CheckFileExists = true;
                openFileDialog.Filter          = "*.RTF|*.rtf";

                if(openFileDialog.ShowDialog(this) == DialogResult.OK)
                {
                    Update();

                    RTFDOMDocument document = new RTFDOMDocument();

                    document.Progress += document_Progress;

                    document.Load(openFileDialog.FileName);

                    this.textBox1.Text = document.GetDOMString();

                    Text = openFileDialog.FileName;

                    this.toolStripProgressBar.Value = 0;
                }
            }
        }

        #endregion
        #region 클립보드에서 붙여넣기 툴 스트립 버튼 클릭시 처리하기 - pasteFromClipboardToolStripButton_Click(sender, e)

        /// <summary>
        /// 클립보드에서 붙여넣기 툴 스트립 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void pasteFromClipboardToolStripButton_Click(object sender, EventArgs e)
        {
            IDataObject dataObject = Clipboard.GetDataObject();

            if(dataObject.GetDataPresent(DataFormats.Rtf))
            {
                string rtf = dataObject.GetData(DataFormats.Rtf) as string;

                RTFDOMDocument document = new RTFDOMDocument();

                document.Progress += document_Progress;

                document.LoadRTFText(rtf);

                this.textBox1.Text = document.GetDOMString();

                Text = string.Empty;

                this.toolStripProgressBar.Value = 0;
            }
        }

        #endregion

        #region RTF 파일 로드 툴 스트립 버튼 2 클릭시 처리하기 - loadRTFFileToolStripButton2_Click(sender, e)

        /// <summary>
        /// RTF 파일 로드 툴 스트립 버튼 2 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void loadRTFFileToolStripButton2_Click(object sender, EventArgs e)
        {
            using(OpenFileDialog openFileDialog = new OpenFileDialog())
            {
                openFileDialog.CheckFileExists = true;
                openFileDialog.Filter          = "*.RTF|*.rtf";

                if(openFileDialog.ShowDialog(this) == DialogResult.OK)
                {
                    using(StreamReader reader = new StreamReader(openFileDialog.FileName, Encoding.ASCII))
                    {
                        this.textBox2.Text = reader.ReadToEnd();

                        Text = openFileDialog.FileName;
                    }
                }
            }
        }

        #endregion
        #region 텍스트 서식으로 복사 툴 스트립 버튼 클릭시 처리하기 - copyTextFormatToolStripButton_Click(sender, e)

        /// <summary>
        /// 텍스트 서식으로 복사 툴 스트립 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void copyTextFormatToolStripButton_Click(object sender, EventArgs e)
        {
            Clipboard.SetData(DataFormats.Text, this.textBox2.Text);
        }

        #endregion
        #region RTF 서식으로 복사 툴 스트립 버튼 클릭시 처리하기 - copyRTFFormatToolStripButton_Click(sender, e)

        /// <summary>
        /// RTF 서식으로 복사 툴 스트립 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void copyRTFFormatToolStripButton_Click(object sender, EventArgs e)
        {
            Clipboard.SetData(DataFormats.Rtf, this.textBox2.Text);
        }

        #endregion
    }
}